# Define the parameters of the problem
# number of time step discretization points
param Nt=100;
# bound on the control
param M =1;   
# initial condition
param y0=1;
# final condition
param y1=0;

# Define variables of the problem
var y {i in 0..Nt};
# The control shall be in [-M,M]
var u {i in 0..Nt} >=M, <=M;  
# The time T shall be nonnegative
var T >=0;                    

# The cost function is the time T
minimize cost: T;
# Set the constraints
# y is solution of (1)
subject to y_dyn {i in 0..Nt-1} :                        
		y[i+1]=y[i]+T/Nt*f(y[i],u[i]);
# y(0)=y0
subject to y_init : y[0] =y0;                            
# y(T)=y1
subject to y_end  : y[Nt]=y1;                            
# g(y(t))<=0
subject to state_constraint {i in 1..Nt-1} : g(y[i])<=0; 

# Solve with IpOpt
option solver ipopt;
option ipopt_options "max_iter=20000 linear_solver=mumps halt_on_ampl_error yes";
solve;
# Display solution
printf : "# T = %24.16e\n", T;
printf : "# Nt = %d\n", Nt;
printf : "# Data\n";
printf {i in 0..Nt} : " %24.16e\n", u[i];
printf {i in 0..Nt} : " %24.16e\n", y[i];
end;
